﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using System;
using HeuristicLab.Problems.BinPacking.Interfaces;
using HeuristicLab.Core;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;

namespace HeuristicLab.Problems.BinPacking.Shapes {
  [Item("PackingShape", "Represents an abstract shape to describe the measures and the positioning of objects related to bin-packing problems.")]
  [StorableClass]
  public abstract class PackingShape<T> : Item, IPackingShape
    where T : class, IPackingDimensions {
    public static Type PositionType {
      get { return typeof(T); }
    }

    public abstract bool EnclosesPoint(T myPosition, T checkedPoint);
    public abstract bool Encloses(T checkedPosition, PackingShape<T> checkedShape);
    public abstract bool Overlaps(T myPosition, T checkedPosition, PackingShape<T> checkedShape);
    public abstract void InitializeFromMeasures(int[] measures);
    public abstract int[] ToArray();
    public abstract int MultipliedMeasures { get; }
    public abstract T Origin { get; }

    protected PackingShape(int[] measures) {
      InitializeFromMeasures(measures);
    }

    protected PackingShape() { }


    [StorableConstructor]
    protected PackingShape(bool deserializing) : base(deserializing) { }
    protected PackingShape(PackingShape<T> original, Cloner cloner)
      : base(original, cloner) {
    }
  }
}
